@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2007 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,37 +18,36 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CatJob.h,v 1.9 2002/01/10 15:30:58 lav Exp $ */
+/* $Id: CatJob.h,v 1.13 2007/04/18 06:54:57 lav Exp $ */
 
 #ifndef CATJOB_H
 #define CATJOB_H
 
 #include "CopyJob.h"
+#include "StatusLine.h"
+#include "OutputJob.h"
 
 class ArgV;
-class FDStream;
 
 class CatJob : public CopyJobEnv
 {
 protected:
-   FDStream *global;
-   ArgV *for_each;
+   JobRef<OutputJob> output;
    bool ascii;
    bool auto_ascii;
-   bool fail_if_broken;
 
    void	NextFile();
 
 public:
    int Do();
    int Done();
-   int AcceptSig(int sig);
+   int ExitCode();
 
-   CatJob(FileAccess *s,FDStream *global,ArgV *args);
-   ~CatJob();
+   CatJob(FileAccess *s,OutputJob *output,ArgV *args);
 
    void Ascii() { ascii=true; }
    void Binary() { ascii=auto_ascii=false; }
+   void ShowRunStatus(const SMTaskRef<StatusLine>&);
 };
 
 #endif /* CATJOB_H */
