@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1999 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1999-2007 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CopyJob.h,v 1.21 2006/07/03 13:23:55 lav Exp $ */
+/* $Id: CopyJob.h,v 1.25 2007/07/26 06:10:53 lav Exp $ */
 
 #ifndef COPYJOB_H
 #define COPYJOB_H
@@ -29,11 +29,12 @@
 
 class CopyJob : public Job
 {
-   FileCopy *c;
+protected:
+   SMTaskRef<FileCopy> c;
    bool done;
-   char *name; // file name
-   char *dispname; // displayed file name
-   char *op;   // command name
+   xstring_c name; // file name
+   xstring_c dispname; // displayed file name
+   xstring_c op;   // command name
    bool no_status;
    bool no_status_on_write;
    bool clear_status_on_write;
@@ -42,7 +43,6 @@ class CopyJob : public Job
 
 public:
    CopyJob(FileCopy *c1,const char *n,const char *op1);
-   ~CopyJob();
 
    void NoStatus() { no_status=true; }
    void NoStatusOnWrite() { no_status_on_write=true; }
@@ -50,7 +50,7 @@ public:
    bool HasStatus() const { return !no_status; }
 
    int Do();
-   int Done();
+   int Done() { return done; }
    int ExitCode();
 
    void SuspendInternal() { c->SuspendSlave(); }
@@ -64,7 +64,7 @@ public:
    bool Error() { return c->Error(); }
    const char *ErrorText() { return c->ErrorText(); }
    double GetTimeSpent() { return c->GetTimeSpent(); }
-   off_t GetBytesCount() { return c->GetBytesCount(); }
+   virtual off_t GetBytesCount() { return c->GetBytesCount(); }
    off_t GetSize() { return c->GetSize(); }
    off_t GetPos()  { return c->GetPos(); }
    float GetRate() { return c->GetRate(); }
@@ -75,12 +75,12 @@ public:
    void SetRangeLimit(off_t lim) { return c->SetRangeLimit(lim); }
    void SetDate(time_t d) { c->SetDate(d); }
    void SetSize(off_t s)   { c->SetSize(s); }
-   FileCopy *GetCopy() { return c; }
-   FileCopyPeer *GetPut() { return c->put; }
-   FileCopyPeer *GetGet() { return c->get; }
+   const SMTaskRef<FileCopy>& GetCopy() { return c; }
+   const SMTaskRef<FileCopyPeer>& GetPut() { return c->put; }
+   const SMTaskRef<FileCopyPeer>& GetGet() { return c->get; }
 
    const char *Status(const StatusLine *s,bool base=false);
-   void ShowRunStatus(StatusLine *s);
+   void ShowRunStatus(const SMTaskRef<StatusLine>&);
    void	PrintStatus(int,const char *);
 
    const char *GetName() const { return name; }
@@ -92,6 +92,12 @@ public:
 
    static const char *FormatBytesTimeRate(off_t bytes,double time);
 };
+class CopyJobCreator
+{
+public:
+   virtual CopyJob *New(FileCopy *c,const char *n,const char *o) const = 0;
+   virtual ~CopyJobCreator() {}
+};
 
 class ArgV;
 class CopyJobEnv : public SessionJob
@@ -105,16 +111,18 @@ protected:
    double time_spent;
    const char *op;
    bool no_status;
-   char *cwd;
+   xstring_c cwd;
    bool cont;
    bool ascii;
-   ArgV *args;
+   Ref<ArgV> args;
 
    virtual void NextFile() = 0;
 
    void SetCopier(FileCopy *c,const char *n);
    void AddCopier(FileCopy *c,const char *n);
 
+   Ref<CopyJobCreator> cj_new;
+
 public:
    int Do();
    int Done();
@@ -125,6 +133,8 @@ public:
    CopyJobEnv(FileAccess *s,ArgV *a,bool c=false);
    ~CopyJobEnv();
 
+   void SetCopyJobCreator(CopyJobCreator *c) { cj_new=c; }
+
    void SayFinalWithPrefix(const char *p);
    void SayFinal() { SayFinalWithPrefix(""); }
    void	PrintStatus(int,const char *);
