@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopy.cc,v 1.145 2008/01/31 07:55:13 lav Exp $ */
+/* $Id: FileCopy.cc,v 1.147 2008/05/22 13:20:35 lav Exp $ */
 
 /* FileCopyPeer behaviour:
     1) when suspended, does nothing
@@ -40,6 +40,7 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <math.h>
+#include <stddef.h>
 #include "FileCopy.h"
 #include "url.h"
 #include "log.h"
@@ -519,6 +520,17 @@ const char *FileCopy::GetPercentDoneStr()
    snprintf(buf,8,"(%d%%) ",pct);
    return buf;
 }
+void FileCopy::RateAdd(int a)
+{
+   rate->Add(a);
+   rate_for_eta->Add(a);
+}
+void FileCopy::RateReset()
+{
+   start_time=now;
+   rate->Reset();
+   rate_for_eta->Reset();
+}
 float FileCopy::GetRate()
 {
    if(!rate->Valid() || !put)
@@ -638,6 +650,12 @@ void FileCopy::LogTransfer()
 // FileCopyPeer implementation
 #undef super
 #define super Buffer
+off_t FileCopyPeer::GetSize()
+{
+   if(size>=0 && pos>size)
+      WantSize();
+   return size;
+}
 void FileCopyPeer::SetSize(off_t s)
 {
    size=s;
@@ -658,6 +676,14 @@ void FileCopyPeer::SetDate(time_t d,int p)
       date_set=false;
 }
 
+void FileCopyPeer::SetRange(const off_t s,const off_t lim)
+{
+   range_start=s;
+   range_limit=lim;
+   if(mode==PUT || range_start>GetPos()+0x4000)
+      Seek(range_start);
+}
+
 bool FileCopyPeer::Done()
 {
    if(Error())
