@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileGlob.cc,v 1.10 2007/09/10 10:00:04 lav Exp $ */
+/* $Id: FileGlob.cc,v 1.13 2008/05/22 13:20:35 lav Exp $ */
 
 #include <config.h>
 #include "trio.h"
@@ -27,6 +27,7 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <fnmatch.h>
+#include <stddef.h>
 
 #include "FileGlob.h"
 #include "misc.h"
@@ -44,15 +45,11 @@ Glob::Glob(FileAccess *s,const char *p)
 
    if(pattern[0]=='~')
    {
-      char *slash=strchr(pattern,'/');
+      const char *slash=strchr(pattern,'/');
       if(slash)
-      {
-	 *slash=0;
-	 inhibit_tilde=HasWildcards(pattern);
-	 *slash='/';
-      }
+	 inhibit_tilde=HasWildcards(xstring::get_tmp(pattern,slash-pattern));
       else
-	 inhibit_tilde=HasWildcards(pattern);;
+	 inhibit_tilde=HasWildcards(pattern);
    }
    if(pattern[0] && !HasWildcards(pattern))
    {
@@ -207,6 +204,8 @@ GlobURL::GlobURL(const FileAccessRef& s,const char *p,type_select t)
 {
    NewGlob(p);
 }
+GlobURL::~GlobURL() {}
+
 FileSet *GlobURL::GetResult()
 {
    FileSet &list=*glob->GetResult();
