@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1998 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1998-2007 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FindJob.h,v 1.9 2001/11/09 13:36:27 lav Exp $ */
+/* $Id: FindJob.h,v 1.23 2007/09/10 10:08:30 lav Exp $ */
 
 #ifndef FINDJOB_H
 #define FINDJOB_H
@@ -26,27 +26,29 @@
 #include "Job.h"
 #include "buffer.h"
 #include "ArgV.h"
+#include "GetFileInfo.h"
+#include "PatternSet.h"
 
 class FinderJob : public SessionJob
 {
-   const char *dir;
+   FileAccessRef my_session;
+   FileAccess::Path orig_init_dir;
+
+   xstring_c dir;
    int errors;
-   ListInfo *li;
+   SMTaskRef<GetFileInfo> li;
 
    class place
       {
 	 friend class FinderJob;
 
-	 char *path;
-	 FileSet *fset;
+	 xstring_c path;
+	 Ref<FileSet> fset;
 
-	 place(const char *p,FileSet *f); // eat f, dup p
-	 ~place();
+	 place(const char *p,FileSet *f) : path(p), fset(f) {}
       };
 
-   place **stack;
-   int stack_ptr;
-   int stack_allocated;
+   RefArray<place> stack;
 
    void Up();
    void Down(const char *d);
@@ -57,16 +59,23 @@ class FinderJob : public SessionJob
 
    bool depth_done;
    unsigned file_info_need;
+   /* In certain circumstances, we can skip a LIST altogether and just
+    * pass argument names on: we don't need anything other than the name
+    * (no other file_info_needs) and we're not recursing (which would imply
+    * needing the type.)  This means arguments that don't actually exist
+    * get passed on; if this is inappropriate (ie for a simple Find),
+    * call ValidateArgs(). */
+   bool validate_args;
 
-   char *exclude;
+   Ref<PatternSet> exclude;
 
 protected:
-   enum state_t { INIT, CD, INFO, LOOP, WAIT, DONE };
+   enum state_t { START_INFO, INFO, LOOP, PROCESSING, WAIT, DONE };
    state_t state;
 
    const char *op;
-   char *start_dir;
-   char *init_dir;
+   FileAccessRefC session;
+   FileAccess::Path init_dir;
 
    enum prf_res { PRF_FATAL, PRF_ERR, PRF_OK, PRF_WAIT, PRF_LATER };
    virtual prf_res ProcessFile(const char *d,const FileInfo *fi);
@@ -83,6 +92,9 @@ protected:
    void NextDir(const char *d);
    const char *GetCur() const { return dir; }
    void Need(unsigned need) { file_info_need=need; }
+   void ValidateArgs() { validate_args=true; }
+
+   bool ProcessingURL() { return session!=SessionJob::session; }
 
 public:
    int Do();
@@ -93,44 +105,29 @@ public:
    FinderJob(FileAccess *s);
    ~FinderJob();
 
-   void ShowRunStatus(StatusLine *sl);
-   void PrintStatus(int v);
+   void ShowRunStatus(const SMTaskRef<StatusLine>&);
+   void PrintStatus(int v,const char *);
 
    void BeQuiet() { quiet=true; }
-   void SetExclude(const char *excl) { xfree(exclude); exclude = xstrdup(excl); }
+   void SetExclude(PatternSet *p) { exclude = p; }
    void set_maxdepth(int _maxdepth) { maxdepth = _maxdepth; }
+
+   void Fg();
+   void Bg();
 };
 
 class FinderJob_List : public FinderJob
 {
-   Buffer *buf;
-   ArgV *args;
+   SMTaskRef<IOBuffer> buf;
+   Ref<ArgV> args;
 protected:
    prf_res ProcessFile(const char *d,const FileInfo *fi);
    void Finish();
 
 public:
    FinderJob_List(FileAccess *s,ArgV *a,FDStream *o);
-   ~FinderJob_List();
 
    int Done() { return FinderJob::Done() && buf->Done(); }
 };
 
-class FinderJob_Cmd : public FinderJob
-{
-public:
-   enum cmd_t { GET, RM };
-   FinderJob_Cmd(FileAccess *s,ArgV *a,cmd_t c);
-   ~FinderJob_Cmd();
-   int Done();
-protected:
-   cmd_t cmd;
-   ArgV *args;
-   char *saved_cwd;
-   bool removing_last;
-
-   prf_res ProcessFile(const char *d,const FileInfo *fi);
-   void Finish();
-};
-
 #endif //FINDJOB_H
