@@ -24,13 +24,14 @@
 
 class FinderJob_Du : public FinderJob
 {
-   IOBuffer *buf;
+   Ref<IOBuffer> buf;
 
    /* We keep traversing deeper than this, but we never print a total
     * past this. */
    int max_print_depth;
    bool print_totals;
    int output_block_size;
+   int human_opts;
    bool all_files;
    bool separate_dirs;
    bool file_count;
@@ -41,16 +42,20 @@ class FinderJob_Du : public FinderJob
 
    void Init(const char *d);
 
-   struct stack {
-      char *dir;
+   struct stack_entry {
+      xstring_c dir;
       long long size;
-   } *size_stack;
-   int stack_ptr;
+      stack_entry(const char *dir) : dir(dir), size(0) {}
+   };
+   RefArray<stack_entry> size_stack;
 
-   ArgV *args;
+   Ref<ArgV> args;
 
    void print_size (long long n_blocks, const char *string);
 
+   // prepends last directory name
+   const char *MakeFileName(const char *n);
+
    void Push (const char *d);
    void Pop();
 
@@ -60,7 +65,7 @@ public:
    int Done();
 
    void PrintTotals() { print_totals=true; }
-   void SetBlockSize(int n) { output_block_size = n; }
+   void SetBlockSize(int n,int ho) { output_block_size = n; human_opts = ho; }
    void PrintDepth(int n) { max_print_depth = n; }
    void AllFiles() { all_files=true; }
    void SeparateDirs() { separate_dirs=true; }
@@ -73,7 +78,6 @@ protected:
    void Finish();
    void Enter(const char *d);
    void Exit();
-
 };
 
-#endif
+#endif // FINDJOBDU_H
