@@ -18,39 +18,19 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FtpListInfo.h,v 1.7 2001/10/25 13:42:56 lav Exp $ */
+/* $Id: FtpListInfo.h,v 1.10 2001/12/05 16:37:13 lav Exp $ */
 
 #ifndef FTPLISTINFO_H
 #define FTPLISTINFO_H
 
-#include "ftpclass.h"
+#include "NetAccess.h"
 
-class FtpListInfo : public ListInfo
+class FtpListInfo : public GenericParseListInfo
 {
-   enum state_t
-      {
-	 INITIAL,
-	 GETTING_LONG_LIST,
-	 GETTING_SHORT_LIST,
-	 GETTING_INFO,
-	 DONE
-      };
-   state_t state;
-   Ftp *session;
-
-   Ftp::fileinfo *get_info;
-   int get_info_cnt;
-
-   class FtpSplitList *slist;
-
+   FileSet *ParseShortList(const char *buf,int len);
 public:
-   static FileSet *ParseFtpLongList(const FileSet &lines,int *err,const char *tz);
-
-public:
-   FtpListInfo(Ftp *session);
-   virtual ~FtpListInfo();
-   int Do();
-   const char *Status();
+   virtual FileSet *Parse(const char *buf,int len);
+   FtpListInfo(FileAccess *session,const char *path) : GenericParseListInfo(session,path) {}
 };
 
 #endif//FTPLISTINFO_H
