@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Http.cc,v 1.230 2007/08/24 15:26:35 lav Exp $ */
+/* $Id: Http.cc,v 1.233 2008/05/16 08:44:59 lav Exp $ */
 
 #include <config.h>
 #include "trio.h"
@@ -26,7 +26,6 @@
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <fcntl.h>
-#include <assert.h>
 #include <errno.h>
 #include <stdarg.h>
 #include <time.h>
@@ -57,7 +56,7 @@ CDECL char *strptime(const char *buf, const char *format, struct tm *tm);
 /* Some status code validation macros: */
 #define H_20X(x)        (((x) >= 200) && ((x) < 300))
 #define H_PARTIAL(x)    ((x) == 206)
-#define H_REDIRECTED(x) (((x) == 301) || ((x) == 302))
+#define H_REDIRECTED(x) (((x) == 301) || ((x) == 302) || ((x) == 303) || ((x) == 307))
 #define H_EMPTY(x)	(((x) == 204) || ((x) == 205))
 #define H_CONTINUE(x)	((x) == 100 || (x) == 102)
 #define H_REQUESTED_RANGE_NOT_SATISFIABLE(x) ((x) == 416)
@@ -309,7 +308,7 @@ void Http::SendMethod(const char *method,const char *efile)
 
       xstring cookie;
       MakeCookie(cookie,hostname,efile+(proxy?url::path_index(efile):0));
-      if(cookie && cookie[0])
+      if(cookie.length()>0)
 	 Send("Cookie: %s\r\n",cookie.get());
    }
 }
@@ -1144,10 +1143,7 @@ int Http::Do()
       state=RECEIVING_HEADER;
       m=MOVED;
       if(mode==STORE)
-      {
-	 assert(rate_limit==0);
 	 rate_limit=new RateLimit(hostname);
-      }
 
    case RECEIVING_HEADER:
       if(send_buf->Error() || recv_buf->Error())
@@ -1988,45 +1984,32 @@ void Http::CookieMerge(xstring &all,const char *cookie_c)
 	 c_value=c_name, c_name=0;
       int c_name_len=xstrlen(c_name);
 
-      char *scan=all.get_non_const();
-      for(;;)
+      for(unsigned i=all.skip_all(0,' '); i<all.length(); i=all.skip_all(i+1,' '))
       {
-	 while(*scan==' ') scan++;
-	 if(*scan==0)
-	    break;
-
-	 char *semicolon=strchr(scan,';');
-	 char *eq=strchr(scan,'=');
+	 const char *scan=all+i;
+	 const char *semicolon=strchr(scan,';');
+	 const char *eq=strchr(scan,'=');
 	 if(semicolon && eq>semicolon)
 	    eq=0;
 	 if((eq==0 && c_name==0)
 	 || (eq-scan==c_name_len && !strncmp(scan,c_name,c_name_len)))
 	 {
 	    // remove old cookie.
-	    const char *m=semicolon?semicolon+1:"";
-	    while(*m==' ') m++;
-	    if(*m==0)
-	    {
-	       while(scan>all && scan[-1]==' ')
-		  scan--;
-	       if(scan>all && scan[-1]==';')
-		  scan--;
-	       *scan=0;
-	    }
+	    if(!semicolon)
+	       all.truncate(i);
 	    else
-	       memmove(scan,m,strlen(m)+1);
+	       all.set_substr(i,all.skip_all(semicolon+1-all,' ')-i,"",0);
 	    break;
 	 }
 	 if(!semicolon)
 	    break;
-	 scan=semicolon+1;
+	 i=semicolon+1-all;
       }
 
       // append cookie.
-      int c_len=strlen(all);
-      while(c_len>0 && all[c_len-1]==' ')
-	 c_len--; // trim
-      all.truncate(c_len);
+      all.rtrim(' ');
+      all.rtrim(';');
+      int c_len=all.length();
       if(c_len>0 && all[c_len-1]!=';')
 	 all.append("; ");
       if(c_name)
