@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2007 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Job.h,v 1.26 2006/06/13 14:35:38 lav Exp $ */
+/* $Id: Job.h,v 1.31 2007/09/10 10:32:06 lav Exp $ */
 
 #ifndef JOB_H
 #define JOB_H
@@ -36,9 +36,8 @@ class Job : public SMTask
 {
    static void SortJobs();
 
-   Job	 *next;
+   Job  *next;
    static Job *chain;
-   int	 waiting_alloc;
 
 protected:
    bool fg;
@@ -50,8 +49,7 @@ public:
    int	 jobno;
    Job	 *parent;
 
-   Job	 **waiting;
-   int	 waiting_num;
+   xarray<Job*> waiting;
 
    void AddWaiting(Job *);
    void RemoveWaiting(Job *);
@@ -69,8 +67,8 @@ public:
 
    void	 AllocJobno();
 
-   virtual void	  PrintStatus(int,const char *prefix="\t") {};
-   virtual void	  ShowRunStatus(StatusLine *);
+   virtual void	PrintStatus(int,const char *prefix="\t") {};
+   virtual void	ShowRunStatus(const SMTaskRef<StatusLine>&);
    void ClearStatus()
       {
 	 const char *empty="";
@@ -84,7 +82,7 @@ public:
    virtual void	  Bg();
    virtual void	  Fg();
 
-   char	 *cmdline;
+   xstring cmdline;
    virtual void ListJobs(int verbose_level,int indent=0);
    void PrintJobTitle(int indent=0,const char *suffix=0);
    void ListOneJob(int verbose,int indent=0,const char *suffix=0);
@@ -95,9 +93,10 @@ public:
    bool WaitsFor(Job *);
    static Job *FindWhoWaitsFor(Job *);
    bool CheckForWaitLoop(Job *parent);
-   int NumAwaitedJobs() { return waiting_num; }
+   int NumAwaitedJobs() { return waiting.count(); }
    Job *FindDoneAwaitedJob();
    void WaitForAllChildren();
+   void AllWaitingFg();
 
    static int NumberOfJobs();
    static Job *FindJob(int n);
@@ -138,15 +137,13 @@ public:
 class SessionJob : public Job
 {
 protected:
-   static void Reuse(FileAccess *s) { SessionPool::Reuse(s); }
-   FileAccess *Clone() const;
-
-   SessionJob(FileAccess *);
-   ~SessionJob();
+   SessionJob(FileAccess *s) : session(s) {}
+   FileAccess *Clone() const { return session->Clone(); }
 
 public:
+   FileAccessRef session;
+
    void PrintStatus(int v,const char *);
-   FileAccess *session;
    const char *GetConnectURL()
       {
 	 if(!session)
@@ -157,4 +154,6 @@ public:
    void Bg();
 };
 
+#define JobRef SMTaskRef // it is basically the same
+
 #endif /* JOB_H */
