@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: LocalAccess.cc,v 1.61 2007/12/28 10:23:10 lav Exp $ */
+/* $Id: LocalAccess.cc,v 1.62 2008/05/22 13:20:36 lav Exp $ */
 
 #include <config.h>
 #include <errno.h>
@@ -27,7 +27,6 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <dirent.h>
-#include <glob.h>
 #include <utime.h>
 
 #include "LocalAccess.h"
@@ -36,6 +35,10 @@
 #include "log.h"
 #include "LocalDir.h"
 
+CDECL_BEGIN
+#include <glob.h>
+CDECL_END
+
 #define FILES_AT_ONCE_STAT 64
 #define FILES_AT_ONCE_READDIR 256
 
