@@ -18,13 +18,14 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: LsCache.h,v 1.19 2006/06/13 14:35:39 lav Exp $ */
+/* $Id: LsCache.h,v 1.22 2007/06/08 11:33:35 lav Exp $ */
 
 #ifndef LSCACHE_H
 #define LSCACHE_H
 
 #include <time.h>
 #include "Cache.h"
+#include "FileAccess.h"
 
 class Buffer;
 class FileAccess;
@@ -32,38 +33,34 @@ class FileAccess;
 class LsCacheEntryLoc
 {
    friend class LsCache;
-   char	 *arg;
-   FileAccess *loc;
+   xstring_c arg;
+   SMTaskRef<FileAccess> loc; // no FileAccessRef -> no reuse here
    int	 mode;
 
 public:
    bool Matches(const FileAccess *p_loc,const char *a,int m);
    LsCacheEntryLoc(const FileAccess *p_loc,const char *a,int m);
-   ~LsCacheEntryLoc();
    int EstimateSize() const { return xstrlen(arg)+(arg!=0); }
    const char *GetClosure() const;
 };
 class LsCacheEntryData
 {
    int	 err_code;
-   char	 *data;
-   int	 data_len;
-   FileSet *afset;    // associated file set
+   xstring data;
+   Ref<FileSet> afset;    // associated file set
 public:
    LsCacheEntryData(int e,const char *d,int l,const FileSet *fs);
-   ~LsCacheEntryData();
    void SetData(int e,const char *d,int l,const FileSet *fs);
    void GetData(int *e,const char **d,int *l,const FileSet **fs);
    const FileSet *GetFileSet(const FileAccess *parser);
-   int EstimateSize() const { return data_len+(afset?afset->EstimateMemory():0); }
+   int EstimateSize() const { return data.length()+(afset?afset->EstimateMemory():0); }
 };
 
 class LsCacheEntry : public CacheEntry, public LsCacheEntryLoc, public LsCacheEntryData
 {
 public:
    int EstimateSize() const;
-   ~LsCacheEntry();
-   LsCacheEntry(CacheEntry *n,const FileAccess *p_loc,const char *a,int m,int e,const char *d,int l,const FileSet *fs);
+   LsCacheEntry(const FileAccess *p_loc,const char *a,int m,int e,const char *d,int l,const FileSet *fs);
 };
 
 class LsCache : public Cache
