@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.cc,v 1.160 2007/11/12 06:29:13 lav Exp $ */
+/* $Id: MirrorJob.cc,v 1.163 2008/05/22 14:50:30 lav Exp $ */
 
 #include <config.h>
 
@@ -41,6 +41,10 @@
 #include "pgetJob.h"
 #include "log.h"
 
+CDECL_BEGIN
+#include "lstat.h"
+CDECL_END
+
 #define set_state(s) do { state=(s); \
    Log::global->Format(11,"mirror(%p) enters state %s\n", this, #s); } while(0)
 #define waiting_num waiting.count()
@@ -1640,7 +1644,7 @@ CMD(mirror)
 	 {
 	    // user wants source dir name appended.
 	    const char *base=basename_ptr(source_dir);
-	    if(base[0]!='/')
+	    if(base[0]!='/' && strcmp(base,basename_ptr(arg)))
 	    {
 	       char *target_dir1=alloca_strdup2(target_dir,strlen(base));
 	       strcat(target_dir1,base);
