@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: MirrorJob.h,v 1.52 2007/09/17 10:39:30 lav Exp $ */
+/* $Id: MirrorJob.h,v 1.53 2008/01/21 13:21:22 lav Exp $ */
 
 #ifndef MIRRORJOB_H
 #define MIRRORJOB_H
@@ -67,8 +67,8 @@ class MirrorJob : public Job
    bool create_target_dir;
    bool	no_target_dir;	   // target directory does not exist (for script_only)
 
-   Ref<ListInfo> source_list_info;
-   Ref<ListInfo> target_list_info;
+   SMTaskRef<ListInfo> source_list_info;
+   SMTaskRef<ListInfo> target_list_info;
 
    xstring_c source_dir;
    xstring_c source_relative_dir;
