@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: NetAccess.cc,v 1.102 2007/10/12 14:06:01 lav Exp $ */
+/* $Id: NetAccess.cc,v 1.105 2008/05/22 14:50:30 lav Exp $ */
 
 #include <config.h>
 
@@ -129,17 +129,15 @@ void NetAccess::KeepAlive(int sock)
 }
 void NetAccess::MinimizeLatency(int sock)
 {
-   int tos;
 #ifdef IP_TOS
-   tos = IPTOS_LOWDELAY;
+   int tos = IPTOS_LOWDELAY;
    setsockopt(sock, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int));
 #endif
 }
 void NetAccess::MaximizeThroughput(int sock)
 {
-   int tos;
 #ifdef IP_TOS
-   tos = IPTOS_THROUGHPUT;
+   int tos = IPTOS_THROUGHPUT;
    setsockopt(sock, IPPROTO_IP, IP_TOS, (char *)&tos, sizeof(int));
 #endif
 }
@@ -169,48 +167,6 @@ void NetAccess::SetSocketMaxseg(int sock)
 #endif
 }
 
-const char *NetAccess::SocketNumericAddress(const sockaddr_u *u)
-{
-#ifdef HAVE_GETNAMEINFO
-   static char buf[NI_MAXHOST];
-   if(getnameinfo(&u->sa,SocketAddrLen(u),buf,sizeof(buf),0,0,NI_NUMERICHOST)<0)
-      return "????";
-   return buf;
-#else
-   static char buf[256];
-   if(u->sa.sa_family!=AF_INET)
-      return "????";
-   unsigned char *a=(unsigned char *)&u->in.sin_addr;
-   sprintf(buf,"%u.%u.%u.%u",a[0],a[1],a[2],a[3]);
-   return buf;
-#endif
-}
-int NetAccess::SocketPort(const sockaddr_u *u)
-{
-   if(u->sa.sa_family==AF_INET)
-      return ntohs(u->in.sin_port);
-#if INET6
-   if(u->sa.sa_family==AF_INET6)
-      return ntohs(u->in6.sin6_port);
-#endif
-   return 0;
-}
-
-socklen_t NetAccess::SocketAddrLen(const sockaddr_u *u)
-{
-   if(u->sa.sa_family==AF_INET)
-      return sizeof(u->in);
-#if INET6
-   if(u->sa.sa_family==AF_INET6)
-      return sizeof(u->in6);
-#endif
-   return sizeof(*u);
-}
-
-#if HAVE_INET_ATON && !HAVE_DECL_INET_ATON
-CDECL int inet_aton(const char *,struct in_addr *);
-#endif
-
 int NetAccess::SocketCreate(int af,int type,int proto)
 {
    int s=socket(af,type,proto);
@@ -223,7 +179,7 @@ int NetAccess::SocketCreate(int af,int type,int proto)
    if(af==AF_INET)
    {
       b=ResMgr::Query("net:socket-bind-ipv4",hostname);
-      if(!(b && b[0] && inet_aton(b,&bind_addr.in.sin_addr)))
+      if(!(b && b[0] && inet_pton(af,b,&bind_addr.in.sin_addr)))
 	 b=0;
    }
 #if INET6
@@ -236,7 +192,7 @@ int NetAccess::SocketCreate(int af,int type,int proto)
 #endif
    if(b)
    {
-      int res=bind(s,&bind_addr.sa,sizeof(bind_addr.sa));
+      int res=bind_addr.bind_to(s);
       if(res==-1)
 	 LogError(0,"bind(socket, %s): %s",b,strerror(errno));
    }
