@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: NetAccess.h,v 1.47 2007/08/08 07:08:32 lav Exp $ */
+/* $Id: NetAccess.h,v 1.49 2008/02/17 08:19:24 lav Exp $ */
 
 #ifndef NETACCESS_H
 #define NETACCESS_H
@@ -66,22 +66,22 @@ protected:
    static void MaximizeThroughput(int sock);
    static void ReuseAddress(int sock);
    static int SocketBuffered(int sock);
-   static const char *SocketNumericAddress(const sockaddr_u *u);
-   static int SocketPort(const sockaddr_u *u);
-   static socklen_t SocketAddrLen(const sockaddr_u *u);
+   static const char *SocketNumericAddress(const sockaddr_u *u) { return u->address(); }
+   static int SocketPort(const sockaddr_u *u) { return u->port(); }
+   static socklen_t SocketAddrLen(const sockaddr_u *u) { return u->addr_len(); }
    static int SocketConnect(int fd,const sockaddr_u *u);
    int SocketCreate(int,int,int);
    int SocketCreateTCP(int);
    int Poll(int fd,int ev);
    int CheckHangup(const struct pollfd *pfd,int num);
 
-   xstring proxy;
-   xstring proxy_port;
-   xstring proxy_user;
-   xstring proxy_pass;
-   xstring proxy_proto;
+   xstring_c proxy;
+   xstring_c proxy_port;
+   xstring_c proxy_user;
+   xstring_c proxy_pass;
+   xstring_c proxy_proto;
 
-   xstring home_auto;
+   xstring_c home_auto;
    void	 PropagateHomeAuto();
    const char *FindHomeAuto();
 
