@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: PtyShell.cc,v 1.11 2005/04/29 10:29:59 lav Exp $ */
+/* $Id: PtyShell.cc,v 1.15 2007/08/02 11:28:00 lav Exp $ */
 
 #include <config.h>
 #include "trio.h"
@@ -34,10 +34,10 @@
 # include <sys/ioctl.h>
 #endif
 #include <termios.h>
+#include <stddef.h>
 
 #include "PtyShell.h"
 #include "lftp_pty.h"
-#include "xmalloc.h"
 #include "SignalHook.h"
 #include "ArgV.h"
 #include "misc.h"
@@ -68,11 +68,7 @@ int PtyShell::getfd()
    if(!tty_name)
    {
       if(!NonFatalError(errno))
-      {
-	 char s[256];
-	 sprintf(s,_("pseudo-tty allocation failed: %s"),strerror(errno));
-	 error_text=xstrdup(s);
-      }
+	 error_text.vset(_("pseudo-tty allocation failed: "),strerror(errno),NULL);
       if(use_pipes)
       {
 	 close(pipe0[0]);
@@ -139,7 +135,7 @@ int PtyShell::getfd()
       {
 	 if(chdir(oldcwd)==-1)
 	 {
-	    fprintf(stderr,_("chdir(%s) failed: %s\n"),oldcwd,strerror(errno));
+	    fprintf(stderr,_("chdir(%s) failed: %s\n"),oldcwd.get(),strerror(errno));
 	    fflush(stderr);
 	    _exit(1);
 	 }
@@ -150,7 +146,7 @@ int PtyShell::getfd()
       putenv((char*)"LANGUAGE=C");
       if(a)
 	 execvp(a->a0(),a->GetVNonConst());
-      execl("/bin/sh","sh","-c",name,(char*)NULL);
+      execl("/bin/sh","sh","-c",name.get(),NULL);
       fprintf(stderr,_("execl(/bin/sh) failed: %s\n"),strerror(errno));
       fflush(stderr);
       _exit(1);
@@ -188,8 +184,7 @@ int PtyShell::getfd()
       fcntl(pipe_out,F_SETFL,O_NONBLOCK);
    }
 
-   xfree(oldcwd);
-   oldcwd=0;
+   oldcwd.set(0);
 
    int info;
    waitpid(pid,&info,WUNTRACED);
@@ -203,7 +198,7 @@ void PtyShell::Init()
 {
    a=0;
    w=0;
-   oldcwd=xgetcwd();
+   oldcwd.set_allocated(xgetcwd());
    pg=0;
    closed=false;
    use_pipes=false;
@@ -213,8 +208,7 @@ void PtyShell::Init()
 
 void PtyShell::SetCwd(const char *cwd)
 {
-   xfree(oldcwd);
-   oldcwd=xstrdup(cwd);
+   oldcwd.set(cwd);
 }
 
 PtyShell::PtyShell(const char *filter)
@@ -228,7 +222,7 @@ PtyShell::PtyShell(ArgV *a1)
 {
    Init();
    a=a1;
-   name=a->Combine();
+   name.set_allocated(a->Combine());
 }
 
 PtyShell::~PtyShell()
@@ -245,7 +239,6 @@ PtyShell::~PtyShell()
       close(pipe_out);
    if(w)
       w->Auto();
-   xfree(oldcwd);
 }
 
 bool PtyShell::Done()
