@@ -24,112 +24,185 @@
 #include <errno.h>
 #include <unistd.h>
 #include <assert.h>
-
-/* Our autoconf test will switch to lib/fnmatch.c if the local fnmatch
- * isn't GNU, so this should be OK. */
-#define _GNU_SOURCE
 #include <fnmatch.h>
+#include <stddef.h>
 
 #include "QueueFeeder.h"
+#include "plural.h"
+#include "misc.h"
 
 const char *QueueFeeder::NextCmd(CmdExec *exec, const char *)
 {
    if(jobs == NULL) return NULL;
-  
+
    /* denext the first job */
    QueueJob *job = grab_job(0);
 
-   int bufsize = 1;
-   buffer = (char *) xrealloc(buffer, bufsize);
-   buffer[0] = 0;
-   
-   if(cur_pwd == NULL || strcmp(cur_pwd, job->pwd)) {
-      bufsize += strlen(job->pwd)*2 + 	/* quote safety */
-	         7;			/* 'cd ""; ' */
-      buffer = (char *) xrealloc(buffer, bufsize);
+   buffer.truncate(0);
 
-      strcat(buffer, "cd \"");
-      CmdExec::unquote(buffer+strlen(buffer), job->pwd);
-      strcat(buffer, "\"; ");
-
-      xfree(cur_pwd);
-      cur_pwd = xstrdup(job->pwd);
+   if(xstrcmp(cur_pwd, job->pwd)) {
+      buffer.vappend("cd \"",CmdExec::unquote(job->pwd),"\"; ",NULL);
+      cur_pwd.set(job->pwd);
    }
 
-   if(cur_lpwd == NULL || strcmp(cur_lpwd, job->lpwd)) {
-      bufsize += strlen(job->lpwd)*2 + 	/* quote safety */
-	         8;			/* 'lcd ""; ' */
-      buffer = (char *) xrealloc(buffer, bufsize);
-
-      strcat(buffer, "lcd \"");
-      CmdExec::unquote(buffer+strlen(buffer), job->lpwd);
-      strcat(buffer, "\"; ");
-
-      xfree(cur_lpwd);
-      cur_lpwd = xstrdup(job->lpwd);
+   if(xstrcmp(cur_lpwd, job->lpwd)) {
+      buffer.vappend("lcd \"",CmdExec::unquote(job->lpwd),"\"; ",NULL);
+      cur_lpwd.set(job->lpwd);
    }
 
-   bufsize += strlen(job->cmd) + 1;
-   buffer = (char *) xrealloc(buffer, bufsize);
-   strcat(buffer, job->cmd);
-   strcat(buffer, "\n");
-
+   buffer.vappend(job->cmd.get(),"\n",NULL);
    delete job;
-
    return buffer;
 }
 
-void QueueFeeder::QueueCmd(const char *cmd, const char *pwd, const char *lpwd, int pos)
+void QueueFeeder::QueueCmd(const char *cmd, const char *pwd, const char *lpwd, int pos, int v)
 {
    QueueJob *job = new QueueJob;
-   job->cmd = xstrdup(cmd);
-   job->pwd = xstrdup(pwd);
-   job->lpwd = xstrdup(lpwd);
+   job->cmd.set(cmd);
+   job->pwd.set(pwd);
+   job->lpwd.set(lpwd);
 
    /* we never want a newline at the end: */
-   if(job->cmd[strlen(job->cmd)-1] == '\n')
-      job->cmd[strlen(job->cmd)-1] = 0;
+   if(last_char(job->cmd) == '\n')
+      job->cmd.truncate(strlen(job->cmd)-1);
 
    insert_jobs(job, jobs, lastjob, pos != -1? get_job(pos): NULL);
+   PrintJobs(job, v, _("Added job$|s$"));
 }
-   
-bool QueueFeeder::DelJob(int from)
+
+/* verbose:
+ * 0, quiet
+ * 1, interactive
+ * 2, verbose (print changes of pwd and lpwd)
+ * PrintRequeue, output to requeue
+ */
+void QueueFeeder::PrintJobs(const QueueJob *job, int v, const char *plur) const
+{
+   if(v < 1)
+      return;
+
+   const char *pwd = 0, *lpwd = 0;
+   if(v == PrintRequeue)
+   {
+      for(const QueueJob *j = job; j; j=j->next)
+      {
+	 if(xstrcmp(pwd, job->pwd))
+	 {
+	    printf("cd \"%s\" &\n", CmdExec::unquote(job->pwd));
+	    pwd = job->pwd;
+	 }
+
+	 if(xstrcmp(lpwd, job->lpwd))
+	 {
+	    printf("lcd \"%s\" &\n", CmdExec::unquote(job->lpwd));
+	    lpwd = job->lpwd;
+	 }
+
+	 printf("queue \"%s\"\n", CmdExec::unquote(j->cmd));
+      }
+      return;
+   }
+
+   int job_count=0;
+   for(const QueueJob *j = job; j; j=j->next)
+      job_count++;
+
+   if(job_count>1)
+      printf("%s:\n", plural(plur,job_count));
+
+   pwd = cur_pwd;
+   lpwd = cur_lpwd;
+
+   int n = 1;
+   for(const QueueJob *j = job; j; j=j->next)
+   {
+      /* Print pwd/lpwd changes when v >= 2.  (This only happens when there's
+       * more than one.) */
+      if(v > 2 && (xstrcmp(pwd, job->pwd)))
+      {
+	 printf("\tcd \"%s\"\n", CmdExec::unquote(job->pwd));
+	 pwd = job->pwd;
+      }
+
+      if(v > 2 && (xstrcmp(lpwd, job->lpwd)))
+      {
+	 printf("\tlcd \"%s\"\n", CmdExec::unquote(job->lpwd));
+	 lpwd = job->lpwd;
+      }
+
+      if(job_count==1)
+	 printf("%s: ", plural(plur,job_count));
+      else
+	 printf("\t%2d. ",n++);
+
+      printf("%s\n", j->cmd.get());
+   }
+}
+
+bool QueueFeeder::DelJob(int from, int v)
 {
    QueueJob *job = grab_job(from);
-   if(!job) return false;
+   if(!job)
+   {
+      if(v > 0)
+      {
+	 if(from == -1 || !jobs)
+	    printf(_("No queued jobs.\n"));
+	 else
+	    printf(_("No queued job #%i.\n"), from+1);
+      }
+      return false;
+   }
+
+   PrintJobs(job, v, _("Deleted job$|s$"));
+
    FreeList(job);
    return true;
 }
 
-bool QueueFeeder::DelJob(const char *cmd)
+bool QueueFeeder::DelJob(const char *cmd, int v)
 {
    QueueJob *job = grab_job(cmd);
-   if(!job) return false;
+   if(!job)
+   {
+      if(v > 0)
+      {
+	 if(!jobs)
+	    printf(_("No queued jobs.\n"));
+	 else
+	    printf(_("No queued jobs match \"%s\".\n"), cmd);
+      }
+      return false;
+   }
+
+   PrintJobs(job, v, _("Deleted job$|s$"));
+
    FreeList(job);
    return true;
 }
 
 /* When moving, grab the insertion pointer *before* pulling out things to
  * move, since doing so will change offsets.  (Note that "to == -1" means
- * "move to the end", not "before the last entry".) 
+ * "move to the end", not "before the last entry".)
  */
-bool QueueFeeder::MoveJob(int from, int to)
+bool QueueFeeder::MoveJob(int from, int to, int v)
 {
    /* Safety: make sure we don't try to move an item before itself. */
    if(from == to) return false;
-   
+
    QueueJob *before = to != -1? get_job(to): NULL;
-   
+
    QueueJob *job = grab_job(from);
    if(job == NULL) return false;
 
+   PrintJobs(job, v, _("Moved job$|s$"));
    assert(job != before);
 
    insert_jobs(job, jobs, lastjob, before);
    return true;
 }
 
-bool QueueFeeder::MoveJob(const char *cmd, int to)
+bool QueueFeeder::MoveJob(const char *cmd, int to, int v)
 {
    QueueJob *before = to != -1? get_job(to): NULL;
 
@@ -140,6 +213,8 @@ bool QueueFeeder::MoveJob(const char *cmd, int to)
    QueueJob *job = grab_job(cmd);
    if(job == NULL) return false;
 
+   PrintJobs(job, v, _("Moved job$|s$"));
+
    insert_jobs(job, jobs, lastjob, before);
    return true;
 }
@@ -185,21 +260,30 @@ QueueFeeder::QueueJob *QueueFeeder::grab_job(const char *cmd)
    QueueJob *j = jobs, *head = NULL, *tail = NULL;
 
    while(j) {
-      QueueJob *next = j->next;
+      QueueJob *match = get_next_match(cmd, j);
+      if(!match) break;
+      j = match->next;
 
-      if(!fnmatch(cmd, j->cmd,FNM_CASEFOLD))  {
-	 /* matches */
-	 unlink_job(j);
-	 insert_jobs(j, head, tail, NULL);
-      }
- 
-      j = next;
+      /* matches */
+      unlink_job(match);
+      insert_jobs(match, head, tail, NULL);
    }
 
    return head;
 }
 
-/* insert a list of jobs before "before", or at the end if before is NULL. 
+QueueFeeder::QueueJob *QueueFeeder::get_next_match(const char *cmd, QueueJob *j)
+{
+   while(j) {
+      if(!fnmatch(cmd, j->cmd,FNM_CASEFOLD))
+	 return j;
+
+      j = j->next;
+   }
+   return 0;
+}
+
+/* insert a list of jobs before "before", or at the end if before is NULL.
  * If before is not NULL, it must be contained between lst_head and lst_tail. */
 void QueueFeeder::insert_jobs(QueueJob *job,
       			      QueueJob *&lst_head,
@@ -229,13 +313,6 @@ void QueueFeeder::insert_jobs(QueueJob *job,
    if(!job->prev) lst_head = job;
 }
 
-QueueFeeder::QueueJob::~QueueJob()
-{
-   xfree(cmd);
-   xfree(pwd);
-   xfree(lpwd);
-}
-
 /* Free a list of jobs (forward only; j should be a head pointer.) */
 void QueueFeeder::FreeList(QueueJob *j)
 {
@@ -249,43 +326,39 @@ void QueueFeeder::FreeList(QueueJob *j)
 QueueFeeder::~QueueFeeder()
 {
    FreeList(jobs);
-
-   xfree(cur_pwd);
-   xfree(cur_lpwd);
-   xfree(buffer);
 }
 
 
-void QueueFeeder::PrintStatus(int v) const
+void QueueFeeder::PrintStatus(int v,const char *prefix) const
 {
    if(jobs == NULL)
       return;
-   
-   printf(_("\tCommands queued:\n"));
+
+   if(v == PrintRequeue)
+   {
+      PrintJobs(jobs, v, "");
+      return;
+   }
+
+   printf("%s%s\n",prefix,_("Commands queued:"));
 
    int n = 1;
 
    const char *pwd = cur_pwd, *lpwd = cur_lpwd;
    for(const QueueJob *job = jobs; job; job = job->next) {
-      if(v<2 && n>4)
+      if(v<2 && n>4 && job->next)
       {
-	 printf("\t%2d. ...\n",n);
+	 printf("%s%2d. ...\n",prefix,n);
 	 break;
       }
-      /* Print pwd/lpwd changes when v >= 2.  Ideally, we should
-       * quote these commands, too; but I really don't want to
-       * add another 15 lines of code to this function, and this
-       * output isn't all that useful for copying and pasting anyway
-       * due to formatting.  I'll do it if someone requests it (or
-       * we get better strings ...)
-       */
-      if(v >= 2 && (!pwd || strcmp(pwd, job->pwd)))
-	 printf("\t    cd %s\n", job->pwd);
-      if(v >= 2 && (!lpwd || strcmp(lpwd, job->lpwd)))
-	 printf("\t    lcd %s\n", job->lpwd);
+      /* Print pwd/lpwd changes when v >= 2. */
+      if(v >= 2 && (xstrcmp(pwd, job->pwd)))
+	 printf("%s    cd %s\n",prefix,job->pwd.get());
+      if(v >= 2 && (xstrcmp(lpwd, job->lpwd)))
+	 printf("%s    lcd %s\n",prefix,job->lpwd.get());
       pwd = job->pwd;
       lpwd = job->lpwd;
 
-      printf("\t%2d. %s\n",n++, job->cmd);
+      printf("%s%2d. %s\n",prefix,n++,job->cmd.get());
    }
 }
