@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2002 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ResMgr.cc,v 1.58 2005/12/27 12:02:38 lav Exp $ */
+/* $Id: ResMgr.cc,v 1.72 2008/05/22 14:50:31 lav Exp $ */
 
 #include <config.h>
 
@@ -33,15 +33,15 @@
 #include <arpa/inet.h>
 #include <errno.h>
 CDECL_BEGIN
-#include <regex.h>
+#include "regex.h"
 CDECL_END
 #include "ResMgr.h"
 #include "SMTask.h"
-#include "xmalloc.h"
 #include "misc.h"
+#include "StringSet.h"
 
 ResMgr::Resource  *ResMgr::chain=0;
-ResDecl		  *ResMgr::type_chain=0;
+ResType		  *ResMgr::type_chain=0;
 
 int ResMgr::VarNameCmp(const char *good_name,const char *name)
 {
@@ -78,15 +78,15 @@ int ResMgr::VarNameCmp(const char *good_name,const char *name)
    return res;
 }
 
-const char *ResMgr::FindVar(const char *name,ResDecl **type)
+const char *ResMgr::FindVar(const char *name,const ResType **type)
 {
-   ResDecl *exact_proto=0;
-   ResDecl *exact_name=0;
+   const ResType *exact_proto=0;
+   const ResType *exact_name=0;
 
    *type=0;
 
    int sub=0;
-   ResDecl *type_scan;
+   const ResType *type_scan;
    for(type_scan=type_chain; type_scan; type_scan=type_scan->next)
    {
       switch(VarNameCmp(type_scan->name,name))
@@ -124,30 +124,32 @@ const char *ResMgr::FindVar(const char *name,ResDecl **type)
    return _("ambiguous variable name");
 }
 
+const ResType *ResMgr::FindRes(const char *name)
+{
+   const ResType *type;
+   const char *msg=FindVar(name,&type);
+   if(msg)
+      return 0;
+   return type;
+}
+
 const char *ResMgr::Set(const char *name,const char *cclosure,const char *cvalue)
 {
    const char *msg;
 
-   ResDecl *type;
+   const ResType *type;
    // find type of given variable
    msg=FindVar(name,&type);
    if(msg)
       return msg;
 
-   char *value=xstrdup(cvalue);
+   xstring_c value(cvalue);
    if(value && type->val_valid && (msg=(*type->val_valid)(&value))!=0)
-   {
-      xfree(value);
       return msg;
-   }
 
-   char *closure=xstrdup(cclosure);
+   xstring_c closure(cclosure);
    if(closure && type->closure_valid && (msg=(*type->closure_valid)(&closure))!=0)
-   {
-      xfree(closure);
-      xfree(value);
       return msg;
-   }
 
    Resource **scan;
    // find the old value
@@ -162,29 +164,22 @@ const char *ResMgr::Set(const char *name,const char *cclosure,const char *cvalue
    if(*scan)
    {
       if(value)
-      {
-	 xfree((*scan)->value);
-	 (*scan)->value=value;
-	 xfree(closure);
-      }
+	 (*scan)->value.set(value);
       else
       {
 	 Resource *to_free=*scan;
 	 *scan=(*scan)->next;
 	 delete to_free;
-	 xfree(closure);
       }
-      SMTask::ReconfigAll(type->name);
+      ResClient::ReconfigAll(type->name);
    }
    else
    {
       if(value)
       {
 	 chain=new Resource(chain,type,closure,value);
-	 SMTask::ReconfigAll(type->name);
+	 ResClient::ReconfigAll(type->name);
       }
-      else
-	 xfree(closure);
    }
    return 0;
 }
@@ -212,10 +207,8 @@ int ResMgr::VResourceCompare(const void *a,const void *b)
 
 char *ResMgr::Format(bool with_defaults,bool only_defaults)
 {
-   char *res;
-
    Resource *scan;
-   ResDecl  *dscan;
+   ResType  *dscan;
 
    int n=0;
    int dn=0;
@@ -241,8 +234,7 @@ char *ResMgr::Format(bool with_defaults,bool only_defaults)
       }
    }
 
-   res=(char*)xmalloc(size+1);
-   char *store=res;
+   xstring res("");
 
    Resource **created=(Resource**)alloca((dn+1)*sizeof(Resource*));
    Resource **c_store=created;
@@ -278,56 +270,52 @@ char *ResMgr::Format(bool with_defaults,bool only_defaults)
 
    for(i=0; i<n; i++)
    {
-      sprintf(store,"set %s",arr[i]->type->name);
-      store+=strlen(store);
-      char *s=arr[i]->closure;
+      res.appendf("set %s",arr[i]->type->name);
+      const char *s=arr[i]->closure;
       if(s)
       {
-	 *store++='/';
+	 res.append('/');
 	 bool par=false;
 	 if(strcspn(s," \t>|;&")!=strlen(s))
 	    par=true;
 	 if(par)
-	    *store++='"';
+	    res.append('"');
 	 while(*s)
 	 {
 	    if(strchr("\"\\",*s))
-	       *store++='\\';
-	    *store++=*s++;
+	       res.append('\\');
+	    res.append(*s++);
 	 }
 	 if(par)
-	    *store++='"';
+	    res.append('"');
       }
-      *store++=' ';
+      res.append(' ');
       s=arr[i]->value;
 
       bool par=false;
       if(*s==0 || strcspn(s," \t>|;&")!=strlen(s))
 	 par=true;
       if(par)
-	 *store++='"';
+	 res.append('"');
       while(*s)
       {
 	 if(strchr("\"\\",*s))
-	    *store++='\\';
-	 *store++=*s++;
+	    res.append('\\');
+	 res.append(*s++);
       }
       if(par)
-	 *store++='"';
-      *store++='\n';
+	 res.append('"');
+      res.append('\n');
    }
-   *store=0;
    for(i=0; i<dn; i++)
       delete created[i];
-   return res;
+   return res.borrow();
 }
 
 char **ResMgr::Generator(void)
 {
-   char **res;
-
    Resource *scan;
-   ResDecl  *dscan;
+   ResType  *dscan;
 
    int n=0;
    int dn=0;
@@ -336,8 +324,7 @@ char **ResMgr::Generator(void)
    for(dscan=type_chain; dscan; dscan=dscan->next)
       dn++;
 
-   res=(char**)xmalloc((n + dn + 1) * sizeof(char*));
-   char **store = res;
+   StringSet res;
 
    Resource **created=(Resource**)alloca((dn+1)*sizeof(Resource*));
    Resource **c_store=created;
@@ -364,18 +351,16 @@ char **ResMgr::Generator(void)
    qsort(arr,n,sizeof(*arr),&ResMgr::VResourceCompare);
 
    for(i=0; i<n; i++)
-      *(store++) = xstrdup(arr[i]->type->name);
-
-   *store=0;
+      res.Append(arr[i]->type->name);
 
    for(i=0; i<dn; i++)
       delete created[i];
-   return res;
+   return res.borrow();
 }
 
-const char *ResMgr::BoolValidate(char **value)
+const char *ResMgr::BoolValidate(xstring_c *value)
 {
-   char *v=*value;
+   const char *v=*value;
    const char *newval=0;
 
    switch(v[0])
@@ -398,21 +383,18 @@ const char *ResMgr::BoolValidate(char **value)
       return _("invalid boolean value");
    }
    if(strcmp(v,newval))
-   {
-      xfree(v);
-      *value=xstrdup(newval);
-   }
+      value->set(newval);
 
    return 0;
 }
 
-const char *ResMgr::TriBoolValidate(char **value)
+const char *ResMgr::TriBoolValidate(xstring_c *value)
 {
    if(!BoolValidate(value))
       return 0;
 
    /* not bool */
-   char *v=*value;
+   const char *v=*value;
    const char *newval=0;
 
    switch(v[0])
@@ -424,17 +406,14 @@ const char *ResMgr::TriBoolValidate(char **value)
    }
 
    if(strcmp(v,newval))
-   {
-      xfree(v);
-      *value=xstrdup(newval);
-   }
+      value->set(newval);
 
    return 0;
 }
 
-const char *ResMgr::NumberValidate(char **value)
+const char *ResMgr::NumberValidate(xstring_c *value)
 {
-   char *v=*value;
+   const char *v=*value;
 
    v+=strspn(v," \t");
 
@@ -446,14 +425,14 @@ const char *ResMgr::NumberValidate(char **value)
    if(n==0)
       return _("invalid number");
 
-   v[n]=0;
+   value->truncate(n+(v-*value));
 
    return 0;
 }
 
-const char *ResMgr::FloatValidate(char **value)
+const char *ResMgr::FloatValidate(xstring_c *value)
 {
-   char *v=*value;
+   const char *v=*value;
 
    int n=0;
 
@@ -461,17 +440,17 @@ const char *ResMgr::FloatValidate(char **value)
    if(1>sscanf(v,"%lf%n",&f,&n))
       return _("invalid floating point number");
 
-   v[n]=0;
+   value->truncate(n);
 
    return 0;
 }
 
-const char *ResMgr::UNumberValidate(char **value)
+const char *ResMgr::UNumberValidate(xstring_c *value)
 {
-   char *v=*value;
+   const char *v=*value;
 
    v+=strspn(v," \t");
-   memmove(*value,v,strlen(v)+1);   // drop leading space
+   value->set(v); // drop leading space
 
    v=*value;
    if(!strncasecmp(v,"0x",2))
@@ -482,11 +461,19 @@ const char *ResMgr::UNumberValidate(char **value)
    if(n==0)
       return _("invalid number");
 
-   v[n]=0;
+   value->truncate(n+(v-*value));
 
    return 0;
 }
 
+ResMgr::Resource::Resource(Resource *next,const ResType *type,const char *closure,const char *value)
+   : type(type), value(value), closure(closure), next(next)
+{
+}
+ResMgr::Resource::~Resource()
+{
+}
+
 bool ResMgr::Resource::ClosureMatch(const char *cl_data)
 {
    if(!closure && !cl_data)
@@ -498,10 +485,8 @@ bool ResMgr::Resource::ClosureMatch(const char *cl_data)
 
 const char *ResMgr::QueryNext(const char *name,const char **closure,Resource **ptr)
 {
-   ResDecl *type;
-   // find type of given variable
-   const char *msg=FindVar(name,&type);
-   if(msg)
+   const ResType *type=FindRes(name);
+   if(!type)
       return 0;
 
    if(*ptr==0)
@@ -521,34 +506,28 @@ const char *ResMgr::QueryNext(const char *name,const char **closure,Resource **p
    return 0;
 }
 
+const char *ResMgr::SimpleQuery(const ResType *type,const char *closure)
+{
+   // find the value
+   for(Resource *scan=chain; scan; scan=scan->next)
+      if(scan->type==type && scan->ClosureMatch(closure))
+	 return scan->value;
+   return 0;
+}
 const char *ResMgr::SimpleQuery(const char *name,const char *closure)
 {
-   const char *msg;
-
-   ResDecl *type;
-   // find type of given variable
-   msg=FindVar(name,&type);
-   if(msg)
+   const ResType *type=FindRes(name);
+   if(!type)
       return 0;
 
-   Resource *scan;
-   // find the value
-   for(scan=chain; scan; scan=scan->next)
-      if(scan->type==type && scan->ClosureMatch(closure))
-	 break;
-
-   // if found
-   if(scan)
-      return scan->value;
-
-   return 0;
+   return SimpleQuery(type,closure);
 }
 
 ResValue ResMgr::Query(const char *name,const char *closure)
 {
    const char *msg;
 
-   ResDecl *type;
+   const ResType *type;
    // find type of given variable
    msg=FindVar(name,&type);
    if(msg)
@@ -558,32 +537,17 @@ ResValue ResMgr::Query(const char *name,const char *closure)
       return 0;
    }
 
-   for(;;)
-   {
-      Resource **scan;
-      // find the value
-      for(scan=&chain; *scan; scan=&(*scan)->next)
-	 if((*scan)->type==type && (*scan)->ClosureMatch(closure))
-	    break;
-      // if found
-      if(*scan)
-	 return (*scan)->value;
-      if(!closure)
-	 break;
-      closure=0;
-   }
-
-   return type->defvalue;
+   return type->Query(closure);
 }
 
-ResValue ResDecl::Query(const char *closure)
+ResValue ResType::Query(const char *closure) const
 {
    const char *v=0;
 
    if(closure)
-      v=ResMgr::SimpleQuery(name,closure);
+      v=ResMgr::SimpleQuery(this,closure);
    if(!v)
-      v=ResMgr::SimpleQuery(name,0);
+      v=ResMgr::SimpleQuery(this,0);
    if(!v)
       v=defvalue;
 
@@ -595,19 +559,35 @@ bool ResMgr::str2bool(const char *s)
    return(strchr("TtYy1+",s[0])!=0 || !strcasecmp(s,"on"));
 }
 
-ResDecl::ResDecl(const char *a_name,const char *a_defvalue,
-		  ResValValid *a_val_valid,ResClValid *a_closure_valid)
+ResDecl::ResDecl(const char *a_name,const char *a_defvalue,ResValValid *a_val_valid,ResClValid *a_closure_valid)
 {
    name=a_name;
-   defvalue=xstrdup(a_defvalue);
+   defvalue=a_defvalue;
    val_valid=a_val_valid;
    closure_valid=a_closure_valid;
-   next=ResMgr::type_chain;
-   ResMgr::type_chain=this;
+   ResMgr::AddType(this);
 }
-ResDecl::~ResDecl()
+ResDecls::ResDecls(ResType *array)
 {
-   for(ResDecl **scan=&ResMgr::type_chain; *scan; scan=&(*scan)->next)
+   while(array->name)
+      ResMgr::AddType(array++);
+}
+ResDecls::ResDecls(ResType *r1,ResType *r2,...)
+{
+   ResMgr::AddType(r1);
+   if(!r2)
+      return;
+   ResMgr::AddType(r2);
+   va_list v;
+   va_start(v,r2);
+   while((r1=va_arg(v,ResType *))!=0)
+      ResMgr::AddType(r1);
+   va_end(v);
+}
+
+ResType::~ResType()
+{
+   for(ResType **scan=&ResMgr::type_chain; *scan; scan=&(*scan)->next)
    {
       if(*scan==this)
       {
@@ -624,9 +604,7 @@ ResDecl::~ResDecl()
       {
 	 if((*scan)->type==this)
 	 {
-	    ResMgr::Resource *to_free=*scan;
-	    *scan=(*scan)->next;
-	    delete to_free;
+	    delete replace_value(*scan,(*scan)->next);
 	    modified=true;
 	 }
 	 else
@@ -639,15 +617,9 @@ ResDecl::~ResDecl()
 	 SMTask::ReconfigAll(this->name);
 #endif
    }
-
-   xfree(defvalue);
 }
 
-#define MINUTE (60)
-#define HOUR   (60*MINUTE)
-#define DAY    (24*HOUR)
-
-void TimeInterval::init(const char *s)
+void TimeIntervalR::init(const char *s)
 {
    double interval=0;
    infty=false;
@@ -692,9 +664,9 @@ void TimeInterval::init(const char *s)
    TimeDiff::Set(interval);
 }
 
-const char *ResMgr::TimeIntervalValidate(char **s)
+const char *ResMgr::TimeIntervalValidate(xstring_c *s)
 {
-   TimeInterval t(*s);
+   TimeIntervalR t(*s);
    if(t.Error())
       return t.ErrorText();
    return 0;
@@ -779,7 +751,7 @@ long long Range::Random()
    return start + (long long)((end-start+1)*mult);
 }
 
-const char *ResMgr::RangeValidate(char **s)
+const char *ResMgr::RangeValidate(xstring_c *s)
 {
    Range r(*s);
    if(r.Error())
@@ -787,7 +759,7 @@ const char *ResMgr::RangeValidate(char **s)
    return 0;
 }
 
-const char *ResMgr::ERegExpValidate(char **s)
+const char *ResMgr::ERegExpValidate(xstring_c *s)
 {
    if(**s==0)
       return 0;
@@ -803,22 +775,18 @@ const char *ResMgr::ERegExpValidate(char **s)
    return 0;
 }
 
-#if HAVE_INET_ATON && !HAVE_DECL_INET_ATON
-CDECL int inet_aton(const char *,struct in_addr *);
-#endif
-
-const char *ResMgr::IPv4AddrValidate(char **value)
+const char *ResMgr::IPv4AddrValidate(xstring_c *value)
 {
    if(!**value)
       return 0;
    struct in_addr addr;
-   if(!inet_aton(*value,&addr))
+   if(!inet_pton(AF_INET,*value,&addr))
       return _("Invalid IPv4 numeric address");
    return 0;
 }
 
 #if INET6
-const char *ResMgr::IPv6AddrValidate(char **value)
+const char *ResMgr::IPv6AddrValidate(xstring_c *value)
 {
    if(!**value)
       return 0;
@@ -829,16 +797,16 @@ const char *ResMgr::IPv6AddrValidate(char **value)
 }
 #endif
 
-const char *ResMgr::FileAccessible(char **value,int mode,int want_dir)
+const char *ResMgr::FileAccessible(xstring_c *value,int mode,int want_dir)
 {
    if(!**value)
       return 0;
    const char *f=expand_home_relative(*value);
-   char *cwd=0;
+   xstring_c cwd;
    const char *error=0;
    if(f[0]!='/')
    {
-      cwd=xgetcwd();
+      cwd.set_allocated(xgetcwd());
       if(cwd)
 	 f=dir_file(cwd,f);
    }
@@ -850,63 +818,101 @@ const char *ResMgr::FileAccessible(char **value,int mode,int want_dir)
    else if(access(f,mode)<0)
       error=strerror(errno);
    else
-   {
-      if(f!=*value)
-      {
-	 xfree(*value);
-	 *value=xstrdup(f);
-      }
-   }
-   xfree(cwd);
+      value->set(f);
    return error;
 }
-const char *ResMgr::FileReadable(char **value)
+const char *ResMgr::FileReadable(xstring_c *value)
 {
    return FileAccessible(value,R_OK);
 }
-const char *ResMgr::FileExecutable(char **value)
+const char *ResMgr::FileExecutable(xstring_c *value)
 {
    return FileAccessible(value,X_OK);
 }
-const char *ResMgr::DirReadable(char **value)
+const char *ResMgr::DirReadable(xstring_c *value)
 {
    return FileAccessible(value,R_OK|X_OK,1);
 }
 
+#ifdef HAVE_ICONV
 CDECL_BEGIN
-#include <iconv.h>
+# include <iconv.h>
 CDECL_END
-const char *ResMgr::CharsetValidate(char **value)
+#endif
+const char *ResMgr::CharsetValidate(xstring_c *value)
 {
    if(!**value)
       return 0;
+#ifdef HAVE_ICONV
    iconv_t ic=iconv_open(*value,*value);
    if(ic==(iconv_t)-1)
       return _("this encoding is not supported");
    iconv_close(ic);
    return 0;
+#else
+   return _("this encoding is not supported");
+#endif
 }
 
-const char *ResMgr::NoClosure(char **)
+const char *ResMgr::NoClosure(xstring_c *)
 {
    return _("no closure defined for this setting");
 }
 
-const char *ResMgr::UNumberPairValidate(char **value)
+const char *ResMgr::UNumberPairValidate(xstring_c *value)
 {
-   char *v=*value;
-
    int n=0;
 
    unsigned a,b;
-   if(2>sscanf(v,"%u%*c%u%n",&a,&b,&n))
+   if(2>sscanf(value->get(),"%u%*c%u%n",&a,&b,&n))
    {
       if(UNumberValidate(value))
 	 return _("invalid pair of numbers");
       return 0;
    }
 
-   v[n]=0;
+   value->truncate(n);
 
    return 0;
 }
+
+ResClient *ResClient::chain;
+ResValue ResClient::Query(const char *name,const char *closure) const
+{
+   if(!strchr(name,':'))
+   {
+      const char *prefix=ResPrefix();
+      char *fullname=(char*)alloca(3+strlen(prefix)+1+strlen(name)+1);
+      sprintf(fullname,"%s:%s",prefix,name);
+      name=fullname;
+   }
+   if(!closure)
+      closure=ResClosure();
+   return ResMgr::Query(name,closure);
+}
+ResClient::ResClient()
+{
+   ListAdd(ResClient,chain,this,next);
+}
+ResClient::~ResClient()
+{
+   ListDel(ResClient,chain,this,next);
+}
+void ResClient::ReconfigAll(const char *r)
+{
+   ListScan(ResClient,chain,next)
+      scan->Reconfig(r);
+}
+
+bool ResType::QueryBool(const char *closure) const
+{
+   return Query(closure).to_bool();
+}
+bool ResMgr::QueryBool(const char *name,const char *closure)
+{
+   return Query(name,closure).to_bool();
+}
+bool ResClient::QueryBool(const char *name,const char *closure) const
+{
+   return Query(name,closure).to_bool();
+}
