@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2008 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SMTask.cc,v 1.51 2007/09/10 10:37:40 lav Exp $ */
+/* $Id: SMTask.cc,v 1.54 2008/05/16 08:45:01 lav Exp $ */
 
 #include <config.h>
 #include <assert.h>
@@ -137,6 +137,12 @@ void SMTask::Delete(SMTask *task)
    if(!task->running && !task->ref_count)
       delete task;
 }
+SMTask *SMTask::_SetRef(SMTask *task,SMTask *new_task)
+{
+   _DeleteRef(task);
+   _MakeRef(new_task);
+   return new_task;
+}
 
 void SMTask::Enter(SMTask *task)
 {
@@ -190,7 +196,7 @@ int SMTask::CollectGarbage()
 	 count++;
 	 if(scan->next)
 	 {
-	    Enter(scan->next); // protect it from deleting
+	    Enter(scan->next); // protect it from deleting (in scan's dtor)
 	    delete scan;
 	    Leave(scan=current);
 	 }
