@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: StatusLine.cc,v 1.36 2006/07/24 08:50:20 lav Exp $ */
+/* $Id: StatusLine.cc,v 1.39 2007/08/02 11:28:00 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
@@ -32,7 +32,6 @@
 #include <stdlib.h>
 #include <mbswidth.h>
 #include "xstring.h"
-#include "xmalloc.h"
 
 #include "ResMgr.h"
 #include "misc.h"
@@ -67,7 +66,6 @@ StatusLine::StatusLine(int new_fd)
    strcpy(def_title,"");
    not_term=!isatty(fd);
    GetWidth();
-   Reconfig();
 }
 
 StatusLine::~StatusLine()
@@ -156,16 +154,15 @@ void StatusLine::WriteTitle(const char *s, int fd) const
 
    const char *status_format = ResMgr::Query("cmd:term-status", getenv("TERM"));
 
-   char *disp;
+   xstring disp;
 
    /* If we have no format, and we have both tsl and fsl, use them: */
    if((!status_format || !*status_format) && to_status_line && from_status_line)
-      disp = xasprintf("%s%s%s", to_status_line, s, from_status_line);
+      disp.vset(to_status_line, s, from_status_line, NULL);
    else
-      disp = Subst(status_format, subst);
+      disp.set_allocated(Subst(status_format, subst));
 
    write(fd, disp, strlen(disp));
-   xfree(disp);
 }
 
 void StatusLine::update(const char *const *newstr,int newstr_height)
