@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1998-2007 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1998-2008 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -155,7 +155,7 @@ void Buffer::vFormat(const char *f, va_list v)
    {
       va_list tmp;
       VA_COPY(tmp,v);
-      int res=vsnprintf(GetSpace(size), size, f, v);
+      int res=vsnprintf(GetSpace(size), size, f, tmp);
       va_end(tmp);
       if(res>=0 && res<size)
       {
@@ -325,9 +325,11 @@ DataRecoder::DataRecoder(const char *from_code,const char *to_code,bool translit
 
 void DirectedBuffer::SetTranslation(const char *enc,bool translit)
 {
-   if(!enc)
+   if(!enc || !*enc)
       return;
    const char *local_code=ResMgr::Query("file:charset",0);
+   if(!local_code || !*local_code)
+      return;
    const char *from_code=(mode==GET?enc:local_code);
    const char *to_code  =(mode==GET?local_code:enc);
    SetTranslator(new DataRecoder(from_code,to_code,translit));
@@ -393,6 +395,10 @@ void IOBuffer::Put(const char *buf,int size)
    }
    DirectedBuffer::Put(buf,size);
 }
+void IOBuffer::Put(const char *buf)
+{
+   Put(buf,strlen(buf));
+}
 
 int IOBuffer::Do()
 {
@@ -728,7 +734,7 @@ void Buffer::PackINT64BE(long long data)
 {
    unsigned long long n;
    if(data<0)
-      n=(unsigned long long)(-data)^0xFFFFFFFFFFFFFFFFULL+1;
+      n=((unsigned long long)(-data)^0xFFFFFFFFFFFFFFFFULL)+1;
    else
       n=(unsigned long long)data;
    PackUINT64BE(n);
@@ -749,7 +755,7 @@ void Buffer::PackINT32BE(int data)
 {
    unsigned n;
    if(data<0)
-      n=(unsigned)(-data)^0xFFFFFFFFU+1;
+      n=((unsigned)(-data)^0xFFFFFFFFU)+1;
    else
       n=(unsigned)data;
    PackUINT32BE(n);
