@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ftpclass.h,v 1.152 2007/10/18 13:27:16 lav Exp $ */
+/* $Id: ftpclass.h,v 1.157 2008/05/16 08:43:27 lav Exp $ */
 
 #ifndef FTPCLASS_H
 #define FTPCLASS_H
@@ -64,13 +64,13 @@ class Ftp : public NetAccess
       CWD_CWD_WAITING_STATE,  // waiting until 'CWD $cwd' finishes
       USER_RESP_WAITING_STATE,// waiting until 'USER $user' finishes
       DATASOCKET_CONNECTING_STATE, // waiting for data_sock to connect
-      WAITING_150_STATE	   // waiting for 150 message
+      WAITING_150_STATE,   // waiting for 150 message
+      WAITING_CCC_SHUTDOWN // waiting for the server to shutdown SSL connection
    };
 
-
    class Connection
    {
-      xstring closure;
+      xstring_c closure;
    public:
       int control_sock;
       SMTaskRef<IOBuffer> control_recv;
@@ -107,17 +107,26 @@ class Ftp : public NetAccess
       bool rest_supported;
       bool site_chmod_supported;
       bool site_utime_supported;
+      bool site_utime2_supported;   // two-argument SITE UTIME
       bool pret_supported;
       bool utf8_supported;
       bool lang_supported;
       bool mlst_supported;
       bool clnt_supported;
       bool host_supported;
+      bool mfmt_supported;
 
       off_t last_rest;	// last successful REST position.
       off_t rest_pos;	// the number sent with REST command.
 
-      Timer abor_close_timer; // timer for closing aborted connection.
+      Timer abor_close_timer;	 // timer for closing aborted connection.
+      Timer stat_timer;		 // timer for sending periodic STAT commands.
+      Timer waiting_150_timer;   // limit time to wait for 150 reply.
+      Timer waiting_ssl_timer;	 // limit time to wait for ssl shutdown
+
+      time_t nop_time;
+      off_t  nop_offset;
+      int    nop_count;
 
 #if USE_SSL
       Ref<lftp_ssl> control_ssl;
@@ -129,11 +138,12 @@ class Ftp : public NetAccess
       bool sscn_on;
       xstring auth_args_supported;
       bool ssl_is_activated() { return control_ssl!=0; }
+      Timer waiting_ssl_shutdown;
 #else
       bool ssl_is_activated() { return false; }
 #endif
 
-      xstring mlst_attr_supported;
+      xstring_c mlst_attr_supported;
 
       Connection(const char *c);
       ~Connection();
@@ -190,12 +200,16 @@ class Ftp : public NetAccess
 	 USER_PROXY,	// check response for USER sent to proxy
 	 PASS,		// check response for PASS
 	 PASS_PROXY,	// check response for PASS sent to proxy
+	 OPEN_PROXY,	// special UserGate proxy command
+	 ACCT_PROXY,	// special ACCT with proxy password
 	 TRANSFER,	// generic check for transfer
 	 TRANSFER_CLOSED, // check for transfer complete when Close()d.
 	 FEAT,
 	 OPTS_UTF8,
 	 LANG,
 	 SITE_UTIME,
+	 SITE_UTIME2,
+	 ALLO,
 	 QUOTED		// check response for any command submitted by QUOTE_CMD
 #if USE_SSL
 	 ,AUTH_TLS,PROT,SSCN,CCC
@@ -203,7 +217,7 @@ class Ftp : public NetAccess
       };
 
       expect_t check_case;
-      xstring arg;
+      xstring_c arg;
       Expect *next;
 
       Expect(expect_t e,const char *a=0) : check_case(e), arg(a) {}
@@ -226,7 +240,7 @@ class Ftp : public NetAccess
       ~ExpectQueue();
 
       void Push(Expect *e);
-      void Push(Expect::expect_t e) { Push(new Expect(e)); }
+      void Push(Expect::expect_t e);
       Expect *Pop();
       Expect *FindLastCWD();
       int Count() { return count; }
@@ -284,30 +298,18 @@ private:
    static const bool ftps; // for convenience
 #endif
 
-   xstring line;
-   xstring all_lines;
+   void	DataAbort();
+   void DataClose();
+   void ControlClose();
 
-   bool	 eof;
-
-   time_t   nop_time;
-   off_t    nop_offset;
-   int	    nop_count;
-
-   Timer retry_timer;
-   Timer stat_timer;	   // timer for sending periodic STAT commands.
-
-   void	 DataAbort();
-   void  DataClose();
-   void  ControlClose();
-
-   void  SendUrgentCmd(const char *cmd);
-   int	 FlushSendQueueOneCmd();
-   int	 FlushSendQueue(bool all=false);
-   void	 SendArrayInfoRequests();
-   void	 SendSiteIdle();
-   void	 SendAcct();
-   void	 SendSiteGroup();
-   void	 SendUTimeRequest();
+   void SendUrgentCmd(const char *cmd);
+   int	FlushSendQueueOneCmd();
+   int	FlushSendQueue(bool all=false);
+   void	SendArrayInfoRequests();
+   void	SendSiteIdle();
+   void	SendAcct();
+   void	SendSiteGroup();
+   void	SendUTimeRequest();
    void SendAuth(const char *auth);
    void TuneConnectionAfterFEAT();
    void SendOPTS_MLST();
@@ -320,13 +322,7 @@ private:
    // response_queue and switch to a state if necessary
    int	 ReceiveResp();
 
-   bool ProxyIsHttp()
-      {
-	 if(!proxy_proto)
-	    return false;
-	 return !strcmp(proxy_proto,"http")
-	     || !strcmp(proxy_proto,"https");
-      }
+   bool ProxyIsHttp();
    int	 http_proxy_status_code;
    // Send CONNECT method to http proxy.
    void	 HttpProxySendConnect();
@@ -340,31 +336,31 @@ private:
    bool	 AbsolutePath(const char *p);
 
    void MoveConnectionHere(Ftp *o);
+   bool GetBetterConnection(int level,bool limit_reached);
+   bool SameConnection(const Ftp *o) const;
 
+   // state
    automate_state state;
+   int flags;
+   bool eof;
+   Timer retry_timer;
 
-   xstring anon_user;
-   xstring anon_pass;
-
-   xstring charset;
-
-   static const char *DefaultAnonPass();
-
-   int	 flags;
+   xstring line;	// last line of last server reply
+   xstring all_lines;   // all lines of last server reply
 
    void	 SetError(int code,const char *mess=0);
 
-   bool  verify_data_address;
-   bool  verify_data_port;
-   bool	 rest_list;
-   xstring list_options;
+   // settings
+   xstring_c anon_user;
+   xstring_c anon_pass;
+   xstring_c charset;
+   xstring_c list_options;
+   int nop_interval;
+   bool verify_data_address;
+   bool verify_data_port;
+   bool	rest_list;
 
-   bool	 GetBetterConnection(int level,bool limit_reached);
-   bool  SameConnection(const Ftp *o) const;
-
-   int	 nop_interval;
-
-   xstring skey_pass;
+   xstring_c skey_pass;
    bool allow_skey;
    bool force_skey;
    const char *make_skey_reply();
