@@ -1,5 +1,5 @@
 
-/*  A Bison parser, made from getdate.y
+/*  A Bison parser, made from /home/lav/lftp/src/getdate.y
     by GNU Bison version 1.28  */
 
 #define YYBISON 1  /* Identify Bison output.  */
@@ -36,7 +36,6 @@
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
-# include "xalloca.h"
 #endif
 
 /* Since the code of getdate.y is not included in the Emacs executable
@@ -199,7 +198,7 @@ static int	yyRelSeconds;
 static int	yyRelYear;
 
 
-#line 179 "getdate.y"
+#line 178 "getdate.y"
 typedef union {
     int			Number;
     enum _MERIDIAN	Meridian;
@@ -282,11 +281,11 @@ static const short yyrhs[] = {    -1,
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-   195,   196,   199,   202,   205,   208,   211,   214,   217,   223,
-   229,   238,   244,   256,   259,   262,   268,   272,   276,   282,
-   286,   304,   310,   316,   320,   325,   329,   336,   344,   347,
-   350,   353,   356,   359,   362,   365,   368,   371,   374,   377,
-   380,   383,   386,   389,   392,   395,   398,   403,   436,   440
+   194,   195,   198,   201,   204,   207,   210,   213,   216,   222,
+   228,   237,   243,   255,   258,   261,   267,   271,   275,   281,
+   285,   303,   309,   315,   319,   324,   328,   335,   343,   346,
+   349,   352,   355,   358,   361,   364,   367,   370,   373,   376,
+   379,   382,   385,   388,   391,   394,   397,   402,   435,   439
 };
 #endif
 
@@ -910,37 +909,37 @@ yyreduce:
   switch (yyn) {
 
 case 3:
-#line 199 "getdate.y"
+#line 198 "getdate.y"
 {
 	    yyHaveTime++;
 	;
     break;}
 case 4:
-#line 202 "getdate.y"
+#line 201 "getdate.y"
 {
 	    yyHaveZone++;
 	;
     break;}
 case 5:
-#line 205 "getdate.y"
+#line 204 "getdate.y"
 {
 	    yyHaveDate++;
 	;
     break;}
 case 6:
-#line 208 "getdate.y"
+#line 207 "getdate.y"
 {
 	    yyHaveDay++;
 	;
     break;}
 case 7:
-#line 211 "getdate.y"
+#line 210 "getdate.y"
 {
 	    yyHaveRel++;
 	;
     break;}
 case 9:
-#line 217 "getdate.y"
+#line 216 "getdate.y"
 {
 	    yyHour = yyvsp[-1].Number;
 	    yyMinutes = 0;
@@ -949,7 +948,7 @@ case 9:
 	;
     break;}
 case 10:
-#line 223 "getdate.y"
+#line 222 "getdate.y"
 {
 	    yyHour = yyvsp[-3].Number;
 	    yyMinutes = yyvsp[-1].Number;
@@ -958,7 +957,7 @@ case 10:
 	;
     break;}
 case 11:
-#line 229 "getdate.y"
+#line 228 "getdate.y"
 {
 	    yyHour = yyvsp[-3].Number;
 	    yyMinutes = yyvsp[-1].Number;
@@ -970,7 +969,7 @@ case 11:
 	;
     break;}
 case 12:
-#line 238 "getdate.y"
+#line 237 "getdate.y"
 {
 	    yyHour = yyvsp[-5].Number;
 	    yyMinutes = yyvsp[-3].Number;
@@ -979,7 +978,7 @@ case 12:
 	;
     break;}
 case 13:
-#line 244 "getdate.y"
+#line 243 "getdate.y"
 {
 	    yyHour = yyvsp[-5].Number;
 	    yyMinutes = yyvsp[-3].Number;
@@ -992,53 +991,53 @@ case 13:
 	;
     break;}
 case 14:
-#line 256 "getdate.y"
+#line 255 "getdate.y"
 {
 	    yyTimezone = yyvsp[0].Number;
 	;
     break;}
 case 15:
-#line 259 "getdate.y"
+#line 258 "getdate.y"
 {
 	    yyTimezone = yyvsp[0].Number - 60;
 	;
     break;}
 case 16:
-#line 263 "getdate.y"
+#line 262 "getdate.y"
 {
 	    yyTimezone = yyvsp[-1].Number - 60;
 	;
     break;}
 case 17:
-#line 268 "getdate.y"
+#line 267 "getdate.y"
 {
 	    yyDayOrdinal = 1;
 	    yyDayNumber = yyvsp[0].Number;
 	;
     break;}
 case 18:
-#line 272 "getdate.y"
+#line 271 "getdate.y"
 {
 	    yyDayOrdinal = 1;
 	    yyDayNumber = yyvsp[-1].Number;
 	;
     break;}
 case 19:
-#line 276 "getdate.y"
+#line 275 "getdate.y"
 {
 	    yyDayOrdinal = yyvsp[-1].Number;
 	    yyDayNumber = yyvsp[0].Number;
 	;
     break;}
 case 20:
-#line 282 "getdate.y"
+#line 281 "getdate.y"
 {
 	    yyMonth = yyvsp[-2].Number;
 	    yyDay = yyvsp[0].Number;
 	;
     break;}
 case 21:
-#line 286 "getdate.y"
+#line 285 "getdate.y"
 {
 	  /* Interpret as YYYY/MM/DD if $1 >= 1000, otherwise as MM/DD/YY.
 	     The goal in recognizing YYYY/MM/DD is solely to support legacy
@@ -1059,7 +1058,7 @@ case 21:
 	;
     break;}
 case 22:
-#line 304 "getdate.y"
+#line 303 "getdate.y"
 {
 	    /* ISO 8601 format.  yyyy-mm-dd.  */
 	    yyYear = yyvsp[-2].Number;
@@ -1068,7 +1067,7 @@ case 22:
 	;
     break;}
 case 23:
-#line 310 "getdate.y"
+#line 309 "getdate.y"
 {
 	    /* e.g. 17-JUN-1992.  */
 	    yyDay = yyvsp[-2].Number;
@@ -1077,14 +1076,14 @@ case 23:
 	;
     break;}
 case 24:
-#line 316 "getdate.y"
+#line 315 "getdate.y"
 {
 	    yyMonth = yyvsp[-1].Number;
 	    yyDay = yyvsp[0].Number;
 	;
     break;}
 case 25:
-#line 320 "getdate.y"
+#line 319 "getdate.y"
 {
 	    yyMonth = yyvsp[-3].Number;
 	    yyDay = yyvsp[-2].Number;
@@ -1092,14 +1091,14 @@ case 25:
 	;
     break;}
 case 26:
-#line 325 "getdate.y"
+#line 324 "getdate.y"
 {
 	    yyMonth = yyvsp[0].Number;
 	    yyDay = yyvsp[-1].Number;
 	;
     break;}
 case 27:
-#line 329 "getdate.y"
+#line 328 "getdate.y"
 {
 	    yyMonth = yyvsp[-1].Number;
 	    yyDay = yyvsp[-2].Number;
@@ -1107,7 +1106,7 @@ case 27:
 	;
     break;}
 case 28:
-#line 336 "getdate.y"
+#line 335 "getdate.y"
 {
 	    yyRelSeconds = -yyRelSeconds;
 	    yyRelMinutes = -yyRelMinutes;
@@ -1118,115 +1117,115 @@ case 28:
 	;
     break;}
 case 30:
-#line 347 "getdate.y"
+#line 346 "getdate.y"
 {
 	    yyRelYear += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 31:
-#line 350 "getdate.y"
+#line 349 "getdate.y"
 {
 	    yyRelYear += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 32:
-#line 353 "getdate.y"
+#line 352 "getdate.y"
 {
 	    yyRelYear += yyvsp[0].Number;
 	;
     break;}
 case 33:
-#line 356 "getdate.y"
+#line 355 "getdate.y"
 {
 	    yyRelMonth += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 34:
-#line 359 "getdate.y"
+#line 358 "getdate.y"
 {
 	    yyRelMonth += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 35:
-#line 362 "getdate.y"
+#line 361 "getdate.y"
 {
 	    yyRelMonth += yyvsp[0].Number;
 	;
     break;}
 case 36:
-#line 365 "getdate.y"
+#line 364 "getdate.y"
 {
 	    yyRelDay += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 37:
-#line 368 "getdate.y"
+#line 367 "getdate.y"
 {
 	    yyRelDay += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 38:
-#line 371 "getdate.y"
+#line 370 "getdate.y"
 {
 	    yyRelDay += yyvsp[0].Number;
 	;
     break;}
 case 39:
-#line 374 "getdate.y"
+#line 373 "getdate.y"
 {
 	    yyRelHour += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 40:
-#line 377 "getdate.y"
+#line 376 "getdate.y"
 {
 	    yyRelHour += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 41:
-#line 380 "getdate.y"
+#line 379 "getdate.y"
 {
 	    yyRelHour += yyvsp[0].Number;
 	;
     break;}
 case 42:
-#line 383 "getdate.y"
+#line 382 "getdate.y"
 {
 	    yyRelMinutes += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 43:
-#line 386 "getdate.y"
+#line 385 "getdate.y"
 {
 	    yyRelMinutes += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 44:
-#line 389 "getdate.y"
+#line 388 "getdate.y"
 {
 	    yyRelMinutes += yyvsp[0].Number;
 	;
     break;}
 case 45:
-#line 392 "getdate.y"
+#line 391 "getdate.y"
 {
 	    yyRelSeconds += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 46:
-#line 395 "getdate.y"
+#line 394 "getdate.y"
 {
 	    yyRelSeconds += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 47:
-#line 398 "getdate.y"
+#line 397 "getdate.y"
 {
 	    yyRelSeconds += yyvsp[0].Number;
 	;
     break;}
 case 48:
-#line 404 "getdate.y"
+#line 403 "getdate.y"
 {
 	    if (yyHaveTime && yyHaveDate && !yyHaveRel)
 	      yyYear = yyvsp[0].Number;
@@ -1259,13 +1258,13 @@ case 48:
 	  ;
     break;}
 case 49:
-#line 437 "getdate.y"
+#line 436 "getdate.y"
 {
 	    yyval.Meridian = MER24;
 	  ;
     break;}
 case 50:
-#line 441 "getdate.y"
+#line 440 "getdate.y"
 {
 	    yyval.Meridian = yyvsp[0].Meridian;
 	  ;
@@ -1492,7 +1491,7 @@ yyerrhandle:
     }
   return 1;
 }
-#line 446 "getdate.y"
+#line 445 "getdate.y"
 
 
 /* Month and day table. */
@@ -1679,15 +1678,21 @@ static TABLE const MilitaryTable[] = {
     { NULL }
 };
 
-
 
 
+static char *get_date_error_buf;
+const char *get_date_error()
+{
+   return get_date_error_buf;
+}
+
 /* ARGSUSED */
 static int
 yyerror (s)
      char *s;
 {
-  fprintf (stderr, "get_date: %s\n", s);
+  free(get_date_error_buf);
+  get_date_error_buf=strdup(s);
   return 0;
 }
 
