@@ -18,14 +18,14 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: log.h,v 1.5 2000/01/27 17:07:55 lav Exp $ */
+/* $Id: log.h,v 1.12 2007/08/19 06:06:38 lav Exp $ */
 
 #ifndef LOG_H
 #define LOG_H
 
 #include <unistd.h>
 #include <stdlib.h>
-#include "StatusLine.h"
+#include <stdarg.h>
 #include "SMTask.h"
 
 class Log : public SMTask
@@ -33,6 +33,10 @@ class Log : public SMTask
    int output;
    bool need_close_output;
    bool tty;
+   bool show_pid;
+   bool show_time;
+   bool show_context;
+   bool at_line_start;
    typedef void (*tty_cb_t)();
    tty_cb_t tty_cb;
 
@@ -43,18 +47,18 @@ class Log : public SMTask
 	 output=-1;
 	 need_close_output=false;
       }
-
-protected:
-   ~Log();
+   bool WillOutput(int l);
 
 public:
-   static Log *global;
+   static SMTaskRef<Log> global;
 
    bool enabled;
    int level;
 
+   void DoWrite(const char *str);
    void Write(int l,const char *str);
    void Format(int l,const char *fmt,...) PRINTF_LIKE(3,4);
+   void vFormat(int l,const char *fmt,va_list v);
 
    void SetLevel(int l) { level=l; }
    void Enable()  { enabled=true;  }
@@ -75,8 +79,16 @@ public:
 
    void Init();
    Log() { Init(); }
+   ~Log();
 
    int Do();
+
+   void ShowPID(bool yes=true) { show_pid=yes; }
+   void ShowTime(bool yes=true) { show_time=yes; }
+   void ShowContext(bool yes=true) { show_context=yes; }
+   void ShowNothing() { show_pid=show_time=show_context=false; }
+
+   static void Cleanup();
 };
 
 #endif // LOG_H
