@@ -18,18 +18,21 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: mgetJob.cc,v 1.24 2007/04/13 12:18:33 lav Exp $ */
+/* $Id: mgetJob.cc,v 1.25 2008/05/22 13:20:43 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <glob.h>
 #include "mgetJob.h"
 #include "misc.h"
 #include "ArgV.h"
 #include "url.h"
 #include "mkdirJob.h"
 
+CDECL_BEGIN
+#include <glob.h>
+CDECL_END
+
 void mgetJob::ShowRunStatus(const SMTaskRef<StatusLine>& s)
 {
    if(rg)
