@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: misc.cc,v 1.91 2007/09/10 09:30:19 lav Exp $ */
+/* $Id: misc.cc,v 1.94 2008/05/22 13:20:44 lav Exp $ */
 
 #include <config.h>
 #include "xmalloc.h"
@@ -42,7 +42,7 @@
 #endif
 
 CDECL_BEGIN
-#include <regex.h>
+#include "regex.h"
 CDECL_END
 #include "misc.h"
 #include "ProcWait.h"
@@ -50,6 +50,7 @@ CDECL_END
 #include "url.h"
 #include "ResMgr.h"
 #include <mbswidth.h>
+#include "strftime.h"
 
 #if HAVE_UNSETENV && !HAVE_DECL_UNSETENV
 CDECL void unsetenv(const char *name);
@@ -181,11 +182,7 @@ const char *expand_home_relative(const char *s)
    {
       // extract user name and find the home
       int name_len=(sl?sl-s-1:strlen(s+1));
-      char *name=(char*)alloca(name_len+1);
-      strncpy(name,s+1,name_len);
-      name[name_len]=0;
-
-      struct passwd *pw=getpwnam(name);
+      struct passwd *pw=getpwnam(xstring::get_tmp(s+1,name_len));
       if(pw)
 	 home=pw->pw_dir;
    }
@@ -730,9 +727,7 @@ xstring& dirname_modify(xstring &ret)
 }
 xstring& dirname(const char *path)
 {
-   xstring& tmp=xstring::get_tmp();
-   tmp.set(path);
-   return dirname_modify(tmp);
+   return dirname_modify(xstring::get_tmp(path));
 }
 
 char last_char(const char *str)
