@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-1997 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2007 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,12 +20,20 @@
 
 #include <config.h>
 #include <errno.h>
-#include <stdio.h>
+#include "trio.h"
 #include "mvJob.h"
+#include "misc.h"
 
 mvJob::mvJob(FileAccess *session,const char *from,const char *to) : SessionJob(session)
 {
    failed=0;
+   if(last_char(to)=='/')
+   {
+      const char *from_bn=basename_ptr(from);
+      char *to1=alloca_strdup2(to,strlen(from_bn));
+      strcat(to1,from_bn);
+      to=to1;
+   }
    session->Rename(from,to);
 }
 
@@ -50,15 +58,15 @@ int mvJob::Do()
    return MOVED;
 }
 
-void  mvJob::PrintStatus(int v)
+void  mvJob::PrintStatus(int v,const char *prefix)
 {
-   SessionJob::PrintStatus(v);
+   SessionJob::PrintStatus(v,prefix);
    if(Done())
       return;
-   printf("\t[%s]\n",session->CurrentStatus());
+   printf("%s[%s]\n",prefix,session->CurrentStatus());
 }
 
-void  mvJob::ShowRunStatus(StatusLine *s)
+void  mvJob::ShowRunStatus(const SMTaskRef<StatusLine>& s)
 {
    if(!Done())
       s->Show("[%s]",session->CurrentStatus());
