@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2004 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2007 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: netrc.cc,v 1.13 2005/12/27 06:25:51 lav Exp $ */
+/* $Id: netrc.cc,v 1.16 2007/08/02 11:27:59 lav Exp $ */
 
 #include <config.h>
 
@@ -27,25 +27,8 @@
 #include <errno.h>
 #include "xstring.h"
 #include "netrc.h"
-#include "xmalloc.h"
 #include "log.h"
 
-NetRC::Entry::Entry(const char *h,const char *u,const char *p,const char *a)
-{
-   host=xstrdup(h);
-   user=xstrdup(u);
-   pass=xstrdup(p);
-   acct=xstrdup(a);
-}
-
-NetRC::Entry::~Entry()
-{
-   xfree(host);
-   xfree(user);
-   xfree(pass);
-   xfree(acct);
-}
-
 static bool comment(const char *s, FILE *f)
 {
    if(s[0]!='#')
