@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: resource.cc,v 1.148 2008/01/21 14:07:02 lav Exp $ */
+/* $Id: resource.cc,v 1.156 2008/05/22 14:48:39 lav Exp $ */
 
 #include <config.h>
 
@@ -35,8 +35,9 @@
 #include "url.h"
 #include "GetPass.h"
 #include "ascii_ctype.h"
-#include "confpaths.h"
+#include "configmake.h"
 #include "misc.h"
+#include "localcharset.h"
 
 static const char *FtpProxyValidate(xstring_c *p)
 {
@@ -59,6 +60,14 @@ static const char *FtpProxyValidate(xstring_c *p)
    return 0;
 }
 
+static const char *FtpProxyAuthTypeValidate(xstring_c *s)
+{
+   if(s->ne("user") && s->ne("joined") && s->ne("joined-acct") && s->ne("open"))
+      // for translator: `user', `joined', `joined-acct', `open' are literals.
+      return _("ftp:proxy-auth-type must be one of: user, joined, joined-acct, open");
+   return 0;
+}
+
 static const char *HttpProxyValidate(xstring_c *p)
 {
    ParsedURL url(*p);
@@ -183,7 +192,7 @@ static ResType lftp_vars[] = {
    {"ftp:port-range",		 "full",  ResMgr::RangeValidate,0},
    {"ftp:port-ipv4",		 "",	  ResMgr::IPv4AddrValidate,0},
    {"ftp:proxy",		 "",	  FtpProxyValidate,0},
-   {"ftp:proxy-auth-joined",	 "no",	  ResMgr::BoolValidate,0},
+   {"ftp:proxy-auth-type",	 "user",  FtpProxyAuthTypeValidate,0},
    {"ftp:rest-list",		 "no",	  ResMgr::BoolValidate,0},
    {"ftp:rest-stor",		 "yes",   ResMgr::BoolValidate,0},
    {"ftp:timezone",		 "GMT",   0,0},
@@ -198,6 +207,7 @@ static ResType lftp_vars[] = {
    {"ftp:ssl-auth",		 "TLS",   AuthArgValidate,0},
    {"ftp:ssl-allow-anonymous",	 "no",	  ResMgr::BoolValidate,0},
    {"ftp:ssl-use-ccc",		 "no",	  ResMgr::BoolValidate,0},
+   {"ftp:ssl-shutdown-timeout",	 "5",	  ResMgr::TimeIntervalValidate,0},
    {"ftp:ssl-data-use-keys",	 "yes",	  ResMgr::BoolValidate,0},
    {"ftps:initial-prot",	 "",	  ProtValidate,0},
 #endif
@@ -214,7 +224,8 @@ static ResType lftp_vars[] = {
    {"ftp:use-pret",		 "yes",   ResMgr::BoolValidate,0},
    {"ftp:use-site-chmod",	 "yes",   ResMgr::BoolValidate,0},
    {"ftp:use-site-idle",	 "no",    ResMgr::BoolValidate,0},
-   {"ftp:use-site-utime",	 "no",	  ResMgr::BoolValidate,0},
+   {"ftp:use-site-utime",	 "yes",	  ResMgr::BoolValidate,0},
+   {"ftp:use-site-utime2",	 "yes",	  ResMgr::BoolValidate,0},
    {"ftp:use-size",		 "yes",   ResMgr::BoolValidate,0},
    {"ftp:use-stat",		 "yes",   ResMgr::BoolValidate,0},
    {"ftp:use-stat-for-list",	 "no",	  ResMgr::BoolValidate,0},
@@ -223,6 +234,7 @@ static ResType lftp_vars[] = {
    {"ftp:verify-address",	 "no",    ResMgr::BoolValidate,0},
    {"ftp:verify-port",		 "no",    ResMgr::BoolValidate,0},
    {"ftp:web-mode",		 "off",   ResMgr::BoolValidate,0},
+   {"ftp:waiting-150-timeout",	 "5",	  ResMgr::TimeIntervalValidate,0},
 #define RETRY_530 \
    "too many|overloaded|try (again |back )?later|is restricted to|"\
    "maximum number|number of connect|only.*session.*allowed|more connection|already connected"
@@ -293,7 +305,7 @@ static ResType lftp_vars[] = {
    {"sftp:connect-program",	 "ssh -a -x",0,0},
    {"sftp:server-program",	 "sftp",  0,0},
    {"sftp:charset",		 "",	  ResMgr::CharsetValidate,0},
-   {"sftp:use-full-path",	 "no",	  0,0},
+   {"sftp:use-full-path",	 "yes",	  ResMgr::BoolValidate,0},
 
    {"file:charset",		 "",	  ResMgr::CharsetValidate,ResMgr::NoClosure},
 
@@ -413,11 +425,9 @@ void ResMgr::ClassInit()
    if(dir_colors)
       Set("color:dir-colors",0,dir_colors);
 
-#if defined(HAVE_NL_LANGINFO) && defined(CODESET)
-   char *cs=nl_langinfo(CODESET);
-   if(cs)
+   const char *cs=locale_charset();
+   if(cs && cs[0])
       Set("file:charset",0,cs);
-#endif
 
    const char *time_style=getenv("TIME_STYLE");
    if(time_style && *time_style)
